print("import: 'atom'")
import atom

print("import: 'atom.catom'")
import atom.catom

print("import: 'atom.datastructures'")
import atom.datastructures

print("import: 'atom.datastructures.sortedmap'")
import atom.datastructures.sortedmap

